@@ -25,6 +25,8 @@
 
 #include <sys/types.h>
 #include <sys/stat.h>
+#include <sys/wait.h>
+#include <stdio.h>
 #include <unistd.h>
 #include <sys/time.h>
 #include <errno.h>
@@ -36,6 +38,9 @@
 
 #define MOUNT_POLL_INTERVAL 3000
 
+#ifdef HAVE_SYS_MNTTAB_H
+#define MNTOPT_RO	"ro"
+#endif
 
 #ifdef HAVE_MNTENT_H
 #include <mntent.h>
@@ -76,6 +81,7 @@ _gnome_vfs_unix_mount_get_unix_device (GnomeVFSUnixMount *mount)
 	fd_set read_fds;
 	struct timeval tv;
 	int res;
+	int status;
 
 	if (pipe (pipes) == -1) {
 		return 0;
@@ -92,13 +98,35 @@ _gnome_vfs_unix_mount_get_unix_device (GnomeVFSUnixMount *mount)
 	if (pid == 0) {
 		/* Child */
 		close (pipes[0]);
-		
-		if (stat (mount->mount_path, &statbuf) == 0) {
-			write (pipes[1], (char *)&statbuf.st_dev, sizeof (dev_t));
+
+		pid = fork ();
+
+		/* Fork an intermediate child that immediately exits so
+		 * we can waitpid it. This means the final process will get
+		 * owned by init and not go zombie
+		 */
+		if (pid == 0) {
+			/* Grandchild */
+			if (stat (mount->mount_path, &statbuf) == 0) {
+				write (pipes[1], (char *)&statbuf.st_dev, sizeof (dev_t));
+			}
+		} else {
+			close (pipes[0]);
+			close (pipes[1]);
 		}
 
 		_exit (0);
 	} else {
+        wait_again:
+		if (waitpid (pid, &status, 0) < 0) {
+			if (errno == EINTR)
+				goto wait_again;
+			else if (errno == ECHILD)
+				; /* do nothing, child already reaped */
+			else
+				g_warning ("waitpid() should not fail in gnome_vfs_unix_mount_get_unix_device");
+		}
+		
 		/* Parent */
 		close (pipes[1]);
 
@@ -119,8 +147,6 @@ _gnome_vfs_unix_mount_get_unix_device (GnomeVFSUnixMount *mount)
 		}
 
 		close (pipes[0]);
-
-		kill (pid, SIGKILL);
 	}
 	
 	return unix_device;
@@ -267,15 +293,15 @@ _gnome_vfs_get_current_unix_mounts (GList **return_list)
 		return TRUE;
 	}
 
-	while ((! getmntent (file, &mntent)) != NULL) {
+	while (! getmntent (file, &mntent)) {
 		mount_entry = g_new0 (GnomeVFSUnixMount, 1);
 
-		mount_entry->mount_path = g_strdup (mntent.mnt_dir);
-		mount_entry->device_path = g_strdup (mntent.mnt_fsname);
-		mount_entry->filesystem_type = g_strdup (mntent.mnt_type);
+		mount_entry->mount_path = g_strdup (mntent.mnt_mountp);
+		mount_entry->device_path = g_strdup (mntent.mnt_special);
+		mount_entry->filesystem_type = g_strdup (mntent.mnt_fstype);
 
 #if defined (HAVE_HASMNTOPT)
-		if (hasmntopt (mntent, MNTOPT_RO) != NULL) {
+		if (hasmntopt (&mntent, MNTOPT_RO) != NULL) {
 			mount_entry->is_read_only = TRUE;
 		}
 #endif
@@ -398,16 +424,6 @@ _gnome_vfs_get_current_unix_mounts (GList **return_list)
 		    	mount_entry->is_read_only = TRUE;
 		}
 
-#ifdef HAVE_SYS_SYSCTL_H
-		if (usermnt != 0) {
-			if (stat (fstab->fs_file, &sb) == 0) {
-				if (sb.st_uid != 0) {
-					mount_entry->is_user_mountable = TRUE;
-				}
-			}
-		}
-#endif
-
 		*return_list = g_list_prepend (*return_list, mount_entry);
 	}
 
@@ -522,21 +538,11 @@ _gnome_vfs_get_unix_mount_table (GList **return_list)
 
 #elif defined (HAVE_SYS_MNTTAB_H)
 
-static char *
-get_fstab_file (void)
-{
-#ifdef _PATH_MNTTAB
-	return _PATH_MNTTAB;
-#else	
-	return "/etc/fstab";
-#endif
-}
-
 gboolean
 _gnome_vfs_get_unix_mount_table (GList **return_list)
 {
 	static time_t last_mtime = 0;
-	struct mntent *mntent;
+	struct mnttab mntent;
 	FILE *file;
 	char *read_file;
 	char *stat_file;
@@ -564,33 +570,33 @@ _gnome_vfs_get_unix_mount_table (GList **return_list)
 		return TRUE;
 	}
 
-	while ((mntent = getmntent (file)) != NULL) {
-		if ((strcmp (mntent->mnt_dir, "ignore") == 0) ||
-		    (strcmp (mntent->mnt_dir, "swap") == 0)) {
+	while (! getmntent (file, &mntent)) {
+		if ((strcmp (mntent.mnt_mountp, "ignore") == 0) ||
+		    (strcmp (mntent.mnt_mountp, "swap") == 0)) {
 			continue;
 		}
 		
 		mount_entry = g_new0 (GnomeVFSUnixMountPoint, 1);
 
-		mount_entry->mount_path = g_strdup (mntent->mnt_dir);
-		mount_entry->device_path = g_strdup (mntent->mnt_fsname);
-		mount_entry->filesystem_type = g_strdup (mntent->mnt_type);
+		mount_entry->mount_path = g_strdup (mntent.mnt_mountp);
+		mount_entry->device_path = g_strdup (mntent.mnt_special);
+		mount_entry->filesystem_type = g_strdup (mntent.mnt_fstype);
 
 
 #ifdef HAVE_HASMNTOPT
-		if (hasmntopt (mntent, MNTOPT_RO) != NULL) {
+		if (hasmntopt (&mntent, MNTOPT_RO) != NULL) {
 			mount_entry->is_read_only = TRUE;
 		}
-		if (hasmntopt (mntent, "loop") != NULL) {
+		if (hasmntopt (&mntent, "lofs") != NULL) {
 			mount_entry->is_loopback = TRUE;
 		}
 #endif
 
-		if ((mntent->mnt_type != NULL && strcmp ("supermount", mntent->mnt_type) == 0)
+		if ((mntent.mnt_fstype != NULL)
 #ifdef HAVE_HASMNTOPT
-		    || hasmntopt (mntent, "user") != NULL
-		    || hasmntopt (mntent, "users") != NULL
-		    || hasmntopt (mntent, "owner") != NULL
+		    || hasmntopt (&mntent, "user") != NULL
+		    || hasmntopt (&mntent, "users") != NULL
+		    || hasmntopt (&mntent, "owner") != NULL
 #endif
 		    ) {
 			mount_entry->is_user_mountable = TRUE;
