@@ -28,6 +28,7 @@
 #include "gnome-vfs-private.h"
 #include "gnome-vfs-volume-monitor.h"
 #include "gnome-vfs-volume-monitor-private.h"
+#include "gnome-vfs-volume-monitor-daemon.h"
 #include "gnome-vfs-volume-monitor-client.h"
 
 static void gnome_vfs_volume_monitor_class_init (GnomeVFSVolumeMonitorClass *klass);
@@ -146,20 +147,6 @@ gnome_vfs_volume_monitor_init (GnomeVFSVolumeMonitor *volume_monitor)
 
 G_LOCK_DEFINE_STATIC (volume_monitor_ref);
 
-/** 
- * gnome_vfs_volume_monitor_ref:
- * @volume_monitor: a #GnomeVFSVolumeMonitor
- *
- * Increases the reference count of a #GnomeVFSVolumeMonitor by one.
- *
- * You shouldn't use this function unless you know what you are doing:
- * #GnomeVFSVolumeMonitor is to be used as a singleton object, see
- * gnome_vfs_get_volume_monitor() for more details.
- *
- * Returns: @volume_monitor with its refcount increased by one.
- *
- * Since: 2.6
- */
 GnomeVFSVolumeMonitor *
 gnome_vfs_volume_monitor_ref (GnomeVFSVolumeMonitor *volume_monitor)
 {
@@ -173,18 +160,6 @@ gnome_vfs_volume_monitor_ref (GnomeVFSVolumeMonitor *volume_monitor)
 	return volume_monitor;
 }
 
-/** 
- * gnome_vfs_volume_monitor_unref:
- * @volume_monitor: a #GnomeVFSVolumeMonitor
- *
- * Decreases the reference count of a #GnomeVFSVolumeMonitor by one.
- *
- * You shouldn't use this function unless you know what you are doing:
- * #GnomeVFSVolumeMonitor is to be used as a singleton object, see
- * gnome_vfs_get_volume_monitor() for more details.
- *
- * Since: 2.6
- */
 void
 gnome_vfs_volume_monitor_unref (GnomeVFSVolumeMonitor *volume_monitor)
 {
@@ -246,7 +221,7 @@ _gnome_vfs_get_volume_monitor_internal (gboolean create)
 	    create &&
 	    !volume_monitor_was_shutdown) {
 		if (gnome_vfs_get_is_daemon ()) {
-			the_volume_monitor = g_object_new (_gnome_vfs_get_daemon_volume_monitor_type (), NULL);
+			the_volume_monitor = g_object_new (GNOME_VFS_TYPE_VOLUME_MONITOR_DAEMON, NULL);
 		} else {
 			the_volume_monitor = g_object_new (GNOME_VFS_TYPE_VOLUME_MONITOR_CLIENT, NULL);
 		}
@@ -257,19 +232,6 @@ _gnome_vfs_get_volume_monitor_internal (gboolean create)
 	return the_volume_monitor;
 }
 
-/** 
- * gnome_vfs_get_volume_monitor:
- *
- * Returns a pointer to the #GnomeVFSVolumeMonitor singleton.
- * #GnomeVFSVolumeMonitor is a singleton, this means it is guaranteed to
- * exist and be valid until gnome_vfs_shutdown() is called. Consequently,
- * it doesn't need to be refcounted since gnome-vfs will hold a reference to 
- * it until it is shut down.
- *
- * Returns: a pointer to the #GnomeVFSVolumeMonitor singleton.
- *
- * Since: 2.6
- */
 GnomeVFSVolumeMonitor *
 gnome_vfs_get_volume_monitor (void)
 {
@@ -282,10 +244,6 @@ _gnome_vfs_volume_monitor_shutdown (void)
 	G_LOCK (the_volume_monitor);
 	
 	if (the_volume_monitor != NULL) {
-		if (!gnome_vfs_get_is_daemon ()) {
-			_gnome_vfs_volume_monitor_client_shutdown (GNOME_VFS_VOLUME_MONITOR_CLIENT (the_volume_monitor));
-		}
-		
 		gnome_vfs_volume_monitor_unref (the_volume_monitor);
 		the_volume_monitor = NULL;
 	}
@@ -295,56 +253,6 @@ _gnome_vfs_volume_monitor_shutdown (void)
 }
 
 
-
-#ifdef USE_HAL
-GnomeVFSVolume *
-_gnome_vfs_volume_monitor_find_volume_by_hal_udi (GnomeVFSVolumeMonitor *volume_monitor,
-						  const char *hal_udi)
-{
-	GList *l;
-	GnomeVFSVolume *vol, *ret;
-
-	/* Doesn't need locks, only called internally on main thread and doesn't write */
-	
-	ret = NULL;
-	for (l = volume_monitor->priv->mtab_volumes; l != NULL; l = l->next) {
-		vol = l->data;
-		if (vol->priv != NULL && vol->priv->hal_udi != NULL && 
-		    vol->priv->activation_uri != NULL && /* Hmm */
-		    strcmp (vol->priv->hal_udi, hal_udi) == 0) {
-			ret = vol;
-			break;
-		}
-	}
-	
-	return ret;
-}
-
-GnomeVFSDrive *
-_gnome_vfs_volume_monitor_find_drive_by_hal_udi (GnomeVFSVolumeMonitor *volume_monitor,
-						 const char           *hal_udi)
-{
-	GList *l;
-	GnomeVFSDrive *drive, *ret;
-
-	/* Doesn't need locks, only called internally on main thread and doesn't write */
-	
-	ret = NULL;
-	for (l = volume_monitor->priv->fstab_drives; l != NULL; l = l->next) {
-		drive = l->data;
-		if (drive->priv != NULL && drive->priv->hal_udi != NULL &&
-		    drive->priv->activation_uri != NULL && /* Hmm */
-		    strcmp (drive->priv->hal_udi, hal_udi) == 0) {
-			ret = drive;
-			break;
-		}
-	}
-
-	return ret;
-}
-#endif /* USE_HAL */
-
-
 GnomeVFSVolume *
 _gnome_vfs_volume_monitor_find_mtab_volume_by_activation_uri (GnomeVFSVolumeMonitor *volume_monitor,
 							      const char *activation_uri)
@@ -411,21 +319,6 @@ _gnome_vfs_volume_monitor_find_connected_server_by_gconf_id (GnomeVFSVolumeMonit
 	return ret;
 }
 
-/** 
- * gnome_vfs_volume_monitor_get_volume_by_id:
- * @volume_monitor: a #GnomeVFSVolumeMonitor
- * @id: the #GnomeVFSVolume id to look for
- *
- * Looks for a #GnomeVFSVolume whose id is @id. A valid @volume_monitor to pass
- * to this function can be acquired using gnome_vfs_get_volume_monitor()
- *
- * Returns: the #GnomeVFSVolume corresponding to @id, or NULL if no 
- * #GnomeVFSVolume with a matching id could be found. The caller owns a 
- * reference on the returned volume, and must call @gnome_vfs_volume_unref
- * when it no longer needs it.
- *
- * Since: 2.6
- */
 GnomeVFSVolume *
 gnome_vfs_volume_monitor_get_volume_by_id (GnomeVFSVolumeMonitor *volume_monitor,
 					   gulong                 id)
@@ -465,21 +358,6 @@ gnome_vfs_volume_monitor_get_volume_by_id (GnomeVFSVolumeMonitor *volume_monitor
 	return NULL;
 }
 
-/** 
- * gnome_vfs_volume_monitor_get_drive_by_id:
- * @volume_monitor: a #GnomeVFSVolumeMonitor
- * @id: the #GnomeVFSVolume id to look for
- *
- * Looks for a #GnomeVFSDrive whose id is @id. A valid @volume_monitor to pass
- * to this function can be acquired using gnome_vfs_get_volume_monitor()
- *
- * Returns: the #GnomeVFSDrive corresponding to @id, or NULL if no 
- * #GnomeVFSDrive with a matching id could be found. The caller owns a 
- * reference on the returned drive, and must call @gnome_vfs_drive_unref
- * when it no longer needs it.
- *
- * Since: 2.6
- */
 GnomeVFSDrive *
 gnome_vfs_volume_monitor_get_drive_by_id  (GnomeVFSVolumeMonitor *volume_monitor,
 					   gulong                 id)
@@ -541,18 +419,9 @@ _gnome_vfs_volume_monitor_disconnect_all (GnomeVFSVolumeMonitor *volume_monitor)
 	}
 }
 
-/** 
- * gnome_vfs_volume_monitor_emit_pre_unmount:
- * @volume_monitor: the #GnomeVFSVolumeMonitor
- * @volume: a #GnomeVFSVolume
- *
- * Emits the "pre-unmount" signal on @volume. 
- *
- * Since: 2.6
- */
 void
-gnome_vfs_volume_monitor_emit_pre_unmount (GnomeVFSVolumeMonitor *volume_monitor,
-					   GnomeVFSVolume        *volume)
+_gnome_vfs_volume_monitor_emit_pre_unmount (GnomeVFSVolumeMonitor *volume_monitor,
+					    GnomeVFSVolume        *volume)
 {
 	g_signal_emit (volume_monitor, volume_monitor_signals[VOLUME_PRE_UNMOUNT], 0, volume);
 }
@@ -604,7 +473,7 @@ _gnome_vfs_volume_monitor_unmounted (GnomeVFSVolumeMonitor *volume_monitor,
 	drive = volume->priv->drive;
 	if (drive != NULL) {
 		_gnome_vfs_volume_unset_drive (volume, drive);
-		_gnome_vfs_drive_remove_volume (drive, volume);
+		_gnome_vfs_drive_unset_volume (drive, volume);
 	}
 	
 	gnome_vfs_volume_unref (volume);
@@ -630,43 +499,24 @@ void
 _gnome_vfs_volume_monitor_disconnected (GnomeVFSVolumeMonitor *volume_monitor,
 					GnomeVFSDrive         *drive)
 {
-	GList *vol_list;
-	GList *current_vol;
+	GnomeVFSVolume *volume;
 	
 	g_mutex_lock (volume_monitor->priv->mutex);
 	volume_monitor->priv->fstab_drives = g_list_remove (volume_monitor->priv->fstab_drives, drive);
 	drive->priv->is_connected = 0;
 	g_mutex_unlock (volume_monitor->priv->mutex);
 
-	vol_list = gnome_vfs_drive_get_mounted_volumes (drive);	
-
-	for (current_vol = vol_list; current_vol != NULL; current_vol = current_vol->next) {  
-		GnomeVFSVolume *volume;
-		volume = GNOME_VFS_VOLUME (vol_list->data);
-
+	volume = drive->priv->volume;
+	if (volume != NULL) {
 		_gnome_vfs_volume_unset_drive (volume, drive);
-		_gnome_vfs_drive_remove_volume (drive, volume);
+		_gnome_vfs_drive_unset_volume (drive, volume);
 	}
-
-	g_list_free (vol_list);
-
+	
 	g_signal_emit (volume_monitor, volume_monitor_signals[DRIVE_DISCONNECTED], 0, drive);
-
+	
 	gnome_vfs_drive_unref (drive);
 }
 
-/** 
- * gnome_vfs_volume_monitor_get_mounted_volumes:
- * @volume_monitor: the #GnomeVFSVolumeMonitor
- *
- * Gets the list of all the mounted #GnomeVFSVolume volumes.
- *
- * Returns: #GList of #GnomeVFSVolume. The #GnomeVFSVolume objects must be 
- * unreffed by the caller when no longer needed with gnome_vfs_volume_unref()
- * and the #GList must be freed.
- *
- * Since: 2.6
- */
 GList *
 gnome_vfs_volume_monitor_get_mounted_volumes (GnomeVFSVolumeMonitor *volume_monitor)
 {
@@ -685,16 +535,6 @@ gnome_vfs_volume_monitor_get_mounted_volumes (GnomeVFSVolumeMonitor *volume_moni
 	return ret;
 }
 
-/** 
- * gnome_vfs_volume_monitor_get_connected_drives:
- * @volume_monitor:
- *
- *
- *
- * Returns:
- *
- * Since: 2.6
- */
 GList *
 gnome_vfs_volume_monitor_get_connected_drives (GnomeVFSVolumeMonitor *volume_monitor)
 {
@@ -801,17 +641,6 @@ _gnome_vfs_volume_monitor_uniquify_drive_name (GnomeVFSVolumeMonitor *volume_mon
 	return unique_name;
 }
 
-/** 
- * gnome_vfs_volume_monitor_get_volume_for_path:
- * @volume_monitor:
- * @path:
- *
- *
- *
- * Returns:
- *
- * Since: 2.6
- */
 GnomeVFSVolume *
 gnome_vfs_volume_monitor_get_volume_for_path  (GnomeVFSVolumeMonitor *volume_monitor,
 					       const char            *path)
